//
// DLL PostLinker.c
//
// Document Author: Thomas Tempelmann (TT)
// Document Contributors: James F. Milne (JFM), Bjrn Eirksson (BE)
// Document Contributors: Patrick Perroud (PP), Nicolas Zinovieff (NZ)
//
// Purpose: Implementation of the RB DLL PostLinker.
//
// Revision History:
//	TT (22 Nov 99): First version, released to public
//	JFM (5 Feb 2000): Added a field to specify the resource ID
//	TT (22 Apr 2000): removed dcon calls; updated the vers rsrc from 1.0 to 1.2
//  BE (9. Oct 2000): Added ability to merge the plugin code to existing plugin
//  PP/NZ (7. jan 2001): Carbonization
//

#define CWPLUGIN_LONG_FILENAME_SUPPORT 0

/* standard headers */
#include <stdio.h>
#include <string.h>

/* system headers */
#if macintosh
#include <Files.h>
#include <Resources.h>
#include <Strings.h>
//#include <dcon.h>
#endif

/* compiler headers */
#include "DropInCompilerLinker.h"
#include "CompilerMapping.h"
#include "CWPluginErrors.h"

/* project headers */
#include "DLL PostLinker Panel.h"

/*
const OSType	outCreator = 'RBv2';
const OSType	outFiletype = 'RBPl';
*/
const OSType	outCreator = 'RSED';
const OSType	outFiletype = 'rsrc';
	


/* prototypes of local functions */
static CWResult	Link(CWPluginContext context);
static CWResult	GetTargetInfo(CWPluginContext context);

/*
 *	main	-	main entry-point for Drop-In Sample linker
 */
pascal short	main(CWPluginContext context)
{
	short		result;
	long		request;
	
	if (CWGetPluginRequest(context, &request) != cwNoErr)
		return cwErrRequestFailed;
	
	result = cwNoErr;
		
	/* dispatch on linker request */
	switch (request) {

	case reqInitLinker:
		/* linker has just been loaded into memory */
		break;
		
	case reqTermLinker:
		/* linker is about to be unloaded from memory */
		break;
		
	case reqLink:
		/* build the final executable */
		result = Link(context);
		break;
	
	case reqTargetInfo:
		/* return info describing target characteristics */
		result = GetTargetInfo(context);
		break;
		
	default:
		result = cwErrRequestFailed;
		break;
	}
	
	result = CWDonePluginRequest(context, result);
	
	return result; 
}

static Boolean FSpExists(FSSpec *file)
{
	FInfo	fndrInfo;
	
	return FSpGetFInfo(file, &fndrInfo) == noErr; 
}


static CWResult	Link(CWPluginContext context)
{
	CWMemHandle		prefsHand;
	DLL_LinkerPref	*prefsPtr;
	DLL_LinkerPref	prefsData;
	CWTargetInfo	targ;
	CWResult		err;
	long			dllFileLen, nameLen, pos;
	Handle			h;
	Handle 			rh;
	FSSpec			resourceSpec;
	short			ph, rph;
	
		// load the relevant prefs

	err = CWGetNamedPreferences(context, kDLLPanelName, &prefsHand);
	if (err) return err;
	err = CWLockMemHandle(context, prefsHand, false, (void**)&prefsPtr);
	if (err) return err;
	prefsData = *prefsPtr;
	err = CWUnlockMemHandle(context, prefsHand);
	if (err) return err;

		// retrieve the linker's output name (which should be a DLL)
	
	err = CWGetTargetInfo (context, &targ);
	if (err) return err;
	
		// 'targ.outfile' now contains the FSSpec of x86 linker's output file
	
		// create a Handle that contains the DLL plus a little header for RB
	
	//dprintf("Opening target file %#s\n", targ.outfile.name);
	if (FSpOpenDF(&targ.outfile, fsRdPerm, &ph) != noErr) return cwErrFileNotFound;
	GetEOF (ph, &dllFileLen);
	if (dllFileLen <= 0) {
		return cwErrFileNotFound;
	}
	nameLen = targ.outfile.name[0];
	h = NewHandleClear (4 + 1 + nameLen + dllFileLen);
	if (!h) {
		FSClose (ph);
		return cwErrOutOfMemory;
	}
	HLock(h);
	pos = 4;
	BlockMoveData (targ.outfile.name, (*h) + pos, 1 + nameLen); 
	pos += 1 + nameLen;
	FSRead (ph, &dllFileLen, (*h) + pos);
	FSClose (ph);

		// now store this handle in a 'PL86' resource
		
	//dprintf("Output file name: %#s\n", prefsData.outfileName);
	FSMakeFSSpec(targ.outfile.vRefNum, targ.outfile.parID, prefsData.outfileName, &resourceSpec);
	
	
	if(prefsData.merge > 0)
	{
		if(!FSpExists(&resourceSpec))
			return cwErrOSError;
			
		rph = FSpOpenResFile(&resourceSpec, fsRdWrPerm);
		if (rph == -1) {
			return cwErrOSError;
		}
		
		rh= GetResource('PL86',prefsData.outresID);
		if(rh !=nil)
			RemoveResource(rh);
		
		ReleaseResource(rh);
	}
	else
	{
		FSpDelete (&resourceSpec);
		FSpCreateResFile (&resourceSpec, outCreator, outFiletype, smSystemScript);
		if (ResError()) {
			return cwErrOSError;
		}
		
		rph = FSpOpenResFile(&resourceSpec, fsRdWrPerm);
		if (rph == -1) {
			return cwErrOSError;
		}
	}
	
	AddResource (h, 'PL86', prefsData.outresID, "\px86 plugin code");
	
	CloseResFile (rph);

/* not necessary:

		// finally, update the link output info

	strncpy ((char*)targ.outfile.name, (char*)prefsData.outfileName, sizeof(targ.outfile.name));
	targ.canRun = false;
	targ.canDebug = false;	
	err = CWSetTargetInfo (context, &targ);
	return err;
*/
	return cwNoErr;
}


static CWResult	GetTargetInfo(CWPluginContext context)
{
	CWTargetInfo	targ;
	CWMemHandle		prefsHand;
	DLL_LinkerPref	prefsData;
	DLL_LinkerPref	*prefsPtr;
	CWResult		err;
	
	memset(&targ, 0, sizeof(targ));
	
	err = CWGetOutputFileDirectory(context, &targ.outfile);

	targ.outputType 		= linkOutputFile;
	targ.linkType			= exelinkageFlat;
	targ.targetCPU			= targetCPUi80x86;
	targ.targetOS			= targetOSWindows;
	
	// load the relevant prefs
	err = CWGetNamedPreferences(context, kDLLPanelName, &prefsHand);
	if (err) return err;
	err = CWLockMemHandle(context, prefsHand, false, (void**)&prefsPtr);
	if (err) return err;
	prefsData = *prefsPtr;
	err = CWUnlockMemHandle(context, prefsHand);
	if (err) return err;

	targ.canRun = false;
	targ.canDebug = false;	
	
	targ.outfileCreator		= outCreator;
	targ.outfileType		= outFiletype;
	//targ.debuggerCreator	= 'CWIE';
	//targ.runHelperCreator	= 'CWIE';

	// we put output file in same folder as project, but with name stored in prefs
	strncpy ((char*)targ.outfile.name, (char*)prefsData.outfileName, sizeof(targ.outfile.name));
	
	targ.runfile = targ.outfile;
	//targ.symfile = targ.outfile;
	
	//targ.linkAgainstFile = targ.outfile;
	
	err = CWSetTargetInfo(context, &targ);
	
	return err;
}

// EOF
