/*
 *  CompilerMapping.h - File Type & Extension => Compiler Mapping for Metrowerks CodeWarrior
 *
 *  Copyright  1995 Metrowerks, Inc.  All rights reserved.
 *	
 */

#ifndef __COMPILERMAPPING_H__
#define __COMPILERMAPPING_H__

#ifdef __MWERKS__
#	pragma once
#endif

#ifndef __CWPLUGINS_H__
#include "CWPlugins.h"
#endif

#ifdef _LATITUDE_
#include <LGAlign.h>
#endif

#ifdef __MWERKS__
#pragma options align=mac68k
#endif

#ifdef	_MSC_VER
#pragma	pack(push,2)
#endif

#ifdef __cplusplus
	extern "C" {
#endif

#ifdef __cplusplus
	#ifdef _LATITUDE_
	const CWDataType Lang_C_CPP		= QUADCONST('c','+','+',' ');
	const CWDataType Lang_Pascal	= QUADCONST('p','a','s','c');
	const CWDataType Lang_Rez		= QUADCONST('r','e','z',' ');
	const CWDataType Lang_Java		= QUADCONST('j','a','v','a');
	const CWDataType Lang_MISC		= QUADCONST('\?','\?','\?','\?');
	#else
	const CWDataType Lang_C_CPP		= 'c++ ';
	const CWDataType Lang_Pascal	= 'pasc';
	const CWDataType Lang_Rez		= 'rez ';
	const CWDataType Lang_Java		= 'java';
	const CWDataType Lang_MISC		= '\?\?\?\?';
	#endif
#else
	#ifdef _LATITUDE_
	#define Lang_C_CPP			QUADCONST('c','+','+',' ')
	#define Lang_Pascal			QUADCONST('p','a','s','c')
	#define Lang_Rez			QUADCONST('r','e','z',' ')
	#define Lang_Java			QUADCONST('j','a','v','a')
	#define Lang_MISC			QUADCONST('\?','\?','\?','\?')
	#else
	#define Lang_C_CPP			'c++ '
	#define Lang_Pascal			'pasc'
	#define Lang_Rez			'rez '
	#define Lang_Java			'java'
	#define Lang_MISC			'\?\?\?\?'
	#endif
#endif

/* Compiler flags, as used in member dropinflags of struct DropInFlags returned by compilers		*/
enum
{
	kGeneratescode   =  1L << 31,		/* this compiler generates code							*/
	kGeneratesrsrcs  =  1L << 30,		/* this compiler generates resources					*/
	kCanpreprocess   =  1L << 29,		/* this compiler can accept a Preprocess request		*/
	kCanprecompile   =  1L << 28,		/* this compiler can accept a Precompile request		*/
	kIspascal        =  1L << 27,		/* this is the pascal compiler							*/
	kCanimport       =  1L << 26,		/* this compiler needs the "Import Weak" popup			*/
	kCandisassemble  =  1L << 25,		/* this compiler can disassemble 						*/
	kPersistent      =  1L << 24,		/* keep the compiler resident except on context switches*/
    kCompAllowDupFileNames = 1L << 23,	/* allow multiple project files with the same name  	*/
    kCompMultiTargAware  =  1L << 22,	/* the compiler can be used with multiple targets		*/
	kIsMPAware		=  1L << 21,		/* the compiler can be run in an MP thread				*/
	kCompUsesTargetStorage = 1L << 20,	/* the compiler keeps storage per target				*/
	kCompEmitsOwnBrSymbols = 1L << 19,	/* browser info includes compiler-specific symbols		*/
	kCompAlwaysReload = 1L << 18,		/* always reload the compiler before request			*/
	kCompRequiresProjectBuildStartedMsg   = 1L << 17,  /*  Compiler listens for  project build started messages     */
	kCompRequiresTargetBuildStartedMsg = 1L << 16,     /*  Compiler listens for  target build started messages      */
	kCompRequiresSubProjectBuildStartedMsg = 1L << 15, /*  Compiler listens for  Sub project build started messages */
	kCompRequiresFileListBuildStartedMsg = 1L << 14    /*  Compiler listens for  filelist build started messages    */  
	/* remaining flags are reserved for future use and should be zero-initialized				                    */
};

/* Compiler mapping flags, used in CompilerMapping.flags & SExtensionMapping.flags		*/
typedef unsigned long CompilerMappingFlags;
enum
{
	kPrecompile = 1L << 31,		/* should this file type be Precompiled?				*/
	kLaunchable = 1L << 30,		/* can this file type be double-clicked on?				*/
	kRsrcfile   = 1L << 29,		/* does this file type contain resources for linking?	*/
	kIgnored    = 1L << 28		/* should files of this type be ignored during Make?	*/
	/* remaining flags are reserved for future use and should be zero-initialized		*/
};

typedef struct CompilerMapping {
	CWDataType		type;				/* file type, e.g. 'TEXT' or 0							*/
	char			extension[32];		/* file extension, e.g. .c/.cp/.pch or ""				*/
	char			compiler[32];		/* name of compiler for this file type or ""			*/
	CompilerMappingFlags flags;			/* see above											*/
} CompilerMapping;

/* Format of data in 'EMap' resource, or as returned by  a compiler's 							*/
/* GetExtensionMapping entry point																*/

typedef struct CWExtensionMapping {
	CWDataType		type;				/* MacOS file type, e.g. 'TEXT' or 0					*/
	char			extension[32];		/* file extension, e.g. .c/.cp/.pch or ""				*/
	CompilerMappingFlags flags;			/* see above											*/
} CWExtensionMapping;

#define kCurrentCWExtMapListVersion	1
#define kCurrentCWExtMapListResourceVersion 1

typedef struct CWExtMapList {
	short				version;
	short				nMappings;
	CWExtensionMapping*	mappings;
} CWExtMapList;

typedef struct CWExtMapListResource {
	short				version;
	short				nMappings;
	CWExtensionMapping	mappings[1];
} CWExtMapListResource;

/* Format of data returned by GetTargetList entry point									*/

#define kCurrentCWTargetListVersion	1
#define kCurrentCWTargetListResourceVersion 1

#ifdef _LATITUDE_
const LG_ALIGN CWTypeList_align[] = {
	LG_ALIGN_SHORT,                                 /* count      */
	LG_ALIGN_ARRAY(LG_ALIGN_UNSIGNED_LONG, 1),      /* CWDataType */
	0
};
#endif

typedef struct CWTypeList {
	short		count;
	CWDataType	items[1];
} CW_CPUList, CW_OSList;

typedef struct CWTargetList {
	short		version;
	short		cpuCount;
	CWDataType*	cpus;
	short		osCount;
	CWDataType*	oss;
} CWTargetList;

#ifdef _LATITUDE_
const LG_ALIGN CWTargetListResource_align[] = {
	LG_ALIGN_SHORT,				/* version 	*/
	LG_ALIGN_STRUCT(CWTypeList_align),	/* cpus		*/
	LG_ALIGN_STRUCT(CWTypeList_align),	/* oss		*/
	0
};
#endif

typedef struct CWTargetListResource {
	short		version;
	CW_CPUList	cpus;
	CW_OSList	oss;
} CWTargetListResource;


#ifdef __cplusplus
	}
#endif

#ifdef	_MSC_VER
#pragma	pack(pop,2)
#endif

#ifdef	__MWERKS__
#pragma options align=reset
#endif

#endif	/* __COMPILERMAPPING_H__ */
