//
// DLL PostLinker Panel.c
//
// Document Author: Thomas Tempelmann (TT)
// Document Contributors: James F. Milne (JFM)
//
// Purpose: Implementation of the preferences panel for the
//	RB DLL PostLinker.
//
// Revision History:
//

/* prevent access to dialog (for a glimpse of the future...) */
#define CW_STRICT_DIALOGS 1

/* standard headers */
#include <stdarg.h>
#include <stdio.h>
#include <string.h>

/* system headers */
#include <AERegistry.h>
#include <Drag.h>
#include <Palettes.h>
#include <Resources.h>
#include <Scrap.h>
#include <TextUtils.h>
#include <Sound.h>

/* compiler headers */
#include <A4Stuff.h>
#include <SetUpA4.h>
#include "DropInPanel.h"

/* project headers */
#include "DLL PostLinker Panel.h"



enum {
	kFactoryPrefsID = 128,
	kCW8ItemListID = 129,
	kOutFileItem = 1,
	kOutResItem = 3
};


/* prototypes of local functions */
static short	InitDialog(PanelParameterBlock *pb);
static void		TermDialog(PanelParameterBlock *pb);
static void		PutData(PanelParameterBlock *pb, Handle options);
static short	GetData(PanelParameterBlock *pb, Handle options, Boolean noisy);
static void		ByteSwapData(DLL_LinkerPref** options);
static short	Filter(PanelParameterBlock *pb, EventRecord *event, short *itemHit);
static void		HandleKey(PanelParameterBlock *pb, EventRecord *event);
static void		HandleClick(PanelParameterBlock *pb, EventRecord *event);
static void		ItemHit(PanelParameterBlock *pb);
static void		FindStatus(PanelParameterBlock *pb);
static void		ObeyCommand(PanelParameterBlock *pb);
static void		Validate(Handle original, Handle current, Boolean *recompile, Boolean *relink, Boolean *reset);
static short	GetFactory(Handle settings);
static short	UpdatePref(Handle settings);
static Boolean	ComparePrefs(Handle prefsHand1, Handle prefsHand2);


/*
 *	main	-	entry-point for Drop-In Preferences Panel
 */
pascal short main(PanelParameterBlock *pb)
{
	short	result, theItem;
	
	EnterCodeResource();
	PrepareCallback();
	
	result = noErr;
			
	switch (pb->request)
	{
	case reqInitPanel:
		/* panel has just been loaded into memory */
		break;

	case reqTermPanel:
		/* panel is about to be unloaded from memory */
		break;

	case reqInitDialog:
		/* hook our dialog item list into the preferences dialog */
		result = InitDialog(pb);
		break;
		
	case reqTermDialog:
		/* unhook our dialog item list from the preferences dialog */
		TermDialog(pb);
		break;
	
	case reqPutData:
		/* put the data in the given handle into our dialog items */
		PutData(pb, pb->currentPrefs);
		break;

	case reqGetData:
		/* fill in the given handle with our dialog items */
		result = GetData(pb, pb->currentPrefs, true);
		break;

	case reqByteSwapData:
		/* byte swap the data in the handle */
		ByteSwapData((DLL_LinkerPref**)pb->currentPrefs);
		break;
		
	case reqFilter:
		/* filter an event in the dialog */
		result = Filter(pb, pb->event, &pb->itemHit);
		break;
		
	case reqItemHit:
		/* handle a hit on one of our dialog items */
		ItemHit(pb);
		break;
		
	case reqDrawCustomItem:
		break;
		
	case reqActivateItem:
		theItem = pb->itemHit - pb->baseItems;
		break;
		
	case reqDeactivateItem:
		theItem = pb->itemHit - pb->baseItems;
		break;
		
	case reqHandleKey:
		theItem = pb->itemHit - pb->baseItems;
		break;
		
	case reqHandleClick:
		theItem = pb->itemHit - pb->baseItems;
		break;
		
	case reqFindStatus:
		FindStatus(pb);
		break;
		
	case reqObeyCommand:
		ObeyCommand(pb);
		break;
	/*
	case reqAEGetPref:
		result = GetPref(pb->prefsKeyword, &pb->prefsDesc, pb->currentPrefs);
		break;

	case reqAESetPref:
		result = SetPref(pb->prefsKeyword, &pb->prefsDesc, pb->currentPrefs);
		break;
	*/
	
	case reqValidate:
		/* determine if we need to reset paths, recompile, or relink */
		Validate(pb->originalPrefs, pb->currentPrefs, &pb->recompile, &pb->relink, &pb->reset);
		break;

	case reqGetFactory:
		/* return our factory settings */
		result = GetFactory(pb->factoryPrefs);
		break;

	case reqUpdatePref:
		/* update the given handle to use the current format for our prefs data */
		result = UpdatePref(pb->currentPrefs);
		break;
		
	default:
		result = paramErr;
		break;
	}
	
	ExitCodeResource();
	
	return (result);
}


static short InitDialog(PanelParameterBlock *pb)
{
	return CWPanlAppendItems(pb, kCW8ItemListID);
}

static void TermDialog(PanelParameterBlock*)
{
}

/*
 *	PutData		-	copy the options data from the handle to the screen
 */
static void PutData(PanelParameterBlock *pb, Handle options)
{
	Str255 resID;
	#define	prefsData	(**(DLL_LinkerPrefHndl)options)
	
	CWPanlSetItemText (pb, kOutFileItem, prefsData.outfileName);
	
	NumToString((long) prefsData.outresID, resID);
	
	CWPanlSetItemText(pb, kOutResItem, resID);
	
	#undef	prefsData
}

/*
 *	GetData		-	copy the options data from screen to the handle
 */
static short GetData(PanelParameterBlock *pb, Handle options, Boolean noisy)
{
	DLL_LinkerPref	prefsData = {PDLL_LINKERPANELCURRENT};
	OSErr		err;
	Str255	resIdText;
	long	num;
	
	err = CWPanlGetItemText(pb, kOutFileItem, prefsData.outfileName, sizeof(prefsData.outfileName));
	if (err || prefsData.outfileName[0]==0) {
		// An error occurred while retrieving the text.  It was probably too 
		// long to fit in the given length.  If we're in "noisy" mode, alert the user.
		if (noisy) {
			SysBeep(8);
			CWPanlActivateItem(pb, kOutFileItem);
			return userCanceledErr;
		}
	}
	
	err = CWPanlGetItemText(pb, kOutResItem, resIdText, sizeof(resIdText));
	if(err || resIdText[0]==0) {
		// We couldn't get the resource ID
		if(noisy) {
			SysBeep(8);
			CWPanlActivateItem(pb, kOutResItem);
			return userCanceledErr;
		}
	}else{
		StringToNum(resIdText, &num);
		prefsData.outresID = num;
	}

	** (DLL_LinkerPrefHndl) options = prefsData;
	
	return (noErr);
}

static void ByteSwapShort(short* x)
{
	union {
		short	s;
		char	c[2];
	}	from,to;

	from.s=*x;
	to.c[0]=from.c[1];
	to.c[1]=from.c[0];
	*x = to.s;
}

/*
 *	ByteSwapData		-	byte-swap the options data
 */
static void ByteSwapData(DLL_LinkerPref** options)
{
	ByteSwapShort(&(**options).version);
}

/*
 *	Filter		-	filter an event for the Preferences panel
 */
static short Filter(PanelParameterBlock *pb, EventRecord *event, short *itemHit)
{
	#pragma unused(pb, event, itemHit)
	return noErr;
}

/*
 *	HandleKey
 */
static void HandleKey(PanelParameterBlock *pb, EventRecord *event)
{
}

/*
 *	HandleClick
 */
static void HandleClick(PanelParameterBlock *pb, EventRecord *event)
{
	pb->itemHit = 0;
}

/*
 *	ItemHit		-	handle an itemHit in a Preferences panel
 */
static void ItemHit(PanelParameterBlock *pb)
{
	short	theItem	= pb->itemHit - pb->baseItems;
	//long	oldValue;
	
	switch (theItem) {
	/*
	case kLinkSymItem:
		CWPanlGetItemValue(pb, theItem, &oldValue);
		CWPanlSetItemValue(pb, theItem, !oldValue);
		break;
	*/
	
	default:
		break;
	}
	
	GetData(pb, pb->currentPrefs, false);
	
	pb->canRevert	= !ComparePrefs(pb->originalPrefs, pb->currentPrefs);
	pb->canFactory	= !ComparePrefs(pb->factoryPrefs,  pb->currentPrefs);
}

/*
 *	FindStatus
 */
static void FindStatus(PanelParameterBlock *pb)
{
	short	command	= pb->itemHit;
	Boolean	enabled	= false;

	pb->itemHit = enabled;
}

/*
 *	ObeyCommand
 */
static void ObeyCommand(PanelParameterBlock *pb)
{
	short		command			= pb->itemHit;
	short		simulatedItem	= 0;
	
	switch (command)
	{
	case menu_Copy:
		break;
	}
	
	pb->itemHit = simulatedItem /* ????? + pb->baseItems */;
}

/*
 *	Validate	-	check if panel's changes require a recompile or relink
 */
static void Validate(Handle original, Handle current, Boolean *recompile, Boolean *relink, Boolean *reset)
{
	#pragma unused(original, current)
	
	*recompile	= false;
	*relink		= false;
	*reset		= false;
}


/*
 *	GetFactory	-	retrieve factory settings
 */
static short GetFactory(Handle settings)
{
	Handle	factory;
	Size	size;
	OSErr	err;
	
	factory = Get1Resource('pref', kFactoryPrefsID);
	if (factory == NULL) {
		err = ResError();
		if (err == noErr)
			err = resNotFound;
		return (err);
	}
	
	size = GetHandleSize(factory);
	SetHandleSize(settings, size);
	err = MemError();
	
	if (!err) {
		BlockMoveData(*factory, *settings, size);
	}
	
	return (err);
}

/*
 *	UpdatePref	-	"upgrade" a pref to the current version
 */
static short UpdatePref(Handle settings)
{
	DLL_LinkerPrefHndl	prefsHand	= (DLL_LinkerPrefHndl) settings;
	OSErr				err		= noErr;
	
	switch ((*prefsHand)->version) {
	case PDLL_LINKERPANEL1: {
		Ptr p = NewPtr(GetHandleSize((Handle)prefsHand));
		BlockMoveData(p, *prefsHand, GetHandleSize((Handle)prefsHand));
		SetHandleSize((Handle)prefsHand, sizeof(DLL_LinkerPref));
		BlockMoveData(*prefsHand, p, GetPtrSize(p));
		(*prefsHand)->outresID = 128;
		break;
	}
	
	case PDLL_LINKERPANELCURRENT:
		break;
		
	default:
		err = kBadPrefVersion;
		break;
	}
	
	return err;
}

/*
 *	ComparePrefs
 */
static Boolean	ComparePrefs(Handle prefsHand1, Handle prefsHand2)
{
	DLL_LinkerPrefHndl	prefs1	= (DLL_LinkerPrefHndl) prefsHand1;
	DLL_LinkerPrefHndl	prefs2	= (DLL_LinkerPrefHndl) prefsHand2;
	
	Boolean ok;
	
	ok =   (GetHandleSize(prefsHand1) == GetHandleSize(prefsHand2) &&
			(*prefs1)->version == (*prefs2)->version &&
			(strncmp((char*)(*prefs1)->outfileName, (char*)(*prefs2)->outfileName, sizeof((*prefs2)->outfileName)-1) == 0) &&
			(*prefs1)->outresID == (*prefs2)->outresID);

	return ok;
}
